// Herding Frosh : PC 111401 ; UVa 10135

// Il s'agit de trouver l'enveloppe convexe des grenouilles, puis de remplacer
// le fil entre deux grenouille successives de cette enveloppe par un fil qui
// fait l'aller-retour jusqu'au poteau, tout en contenant toutes les
// grenouilles. Il faut trouver la meilleure paire de grenouille entre
// laquelle effectuer cet aller-retour ; on calcule simplement la longeur de
// fil ncessaire pour chaque paire de grenouilles successives. C'est fait
// encore par un algo d'enveloppe convexe, modifi pour calculer les longueurs
//  chaque tape.

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <string.h>
#include <math.h>

#define MAX_FROSH 1000

// incrmenter et dcrmenter modulo d
#define INCR(g) g>=d-1?g=1:++g
#define DECR(g) g<=1?g=d-1:--g

// pour stocker les grenouilles
struct gren{
  int numero; // numero dans le tableau non tri
  float x, y; // coordonnes de la grenouille
};


int n; // nombre de grenouilles
gren grenouilles[MAX_FROSH+1]; // position des grenouilles ; indice 0 : poteau
gren grenouilles_triees[MAX_FROSH+1]; // tableau tri
int ec[MAX_FROSH+1]; // numro des grenouilles de l'enveloppe convexe

// pour le tri : point de rfrence
float ref_x, ref_y;

// retourne le signe d'un float
int signe(float x)
{
  if (x < 0)
    return -1;
  if (x == 0)
    return 0;
  return 1;
}

// pour qsort
int comparaison(const void * g1, const void * g2)
{
  float x1 = ((gren *) g1)->x - ref_x, x2 = ((gren *) g2)->x - ref_x;
  float y1 = ((gren *) g1)->y - ref_y, y2 = ((gren *) g2)->y - ref_y;

  // le point de rfrence est le plus petit
  if ((x1==0) && (y1==0))
    return -1;
  if ((x2==0) && (y2==0))
    return 1;

  // les deux sont sur l'axe des ordonnes
  if ((x1==0) && (x2==0))
    if (y1*y2>0)
      return 0; // return signe(y1-y2);
    else
      return signe(y1);

  // une abscisse est nulle
  if ((x1 == 0) && (y1 <= 0))
    return -1;
  if ((x2 == 0) && (y2 <= 0))
    return 1;
  if (x1*x2 == 0)
    return signe(x2-x1);
  /*
  if ((x1 == 0) && (y1 > 0))
    return x2;

  if ((x2 == 0) && (y2 > 0))
    return -x1;
  */

  // ils sont dans des demi-plans diffrents
  if (x1*x2 < 0)
    return signe(x2);

  // cas "gnral"
  return signe(x2*y1-x1*y2);
}

// calcule la distance euclidienne entre deux points
float distance(const gren *g1, const gren *g2)
{
  float delta_x = g1->x - g2->x;
  float delta_y = g1->y - g2->y;
  return sqrt(delta_x*delta_x + delta_y*delta_y);
}

// supprime les points aligns non extrmaux
int suppression()
{
  int d = 0; // point en cours
  for (int i=1; i<=n; ++i)
    {
      if (comparaison(&grenouilles_triees[d], &grenouilles_triees[i]) == 0)
	{
	  if (distance(&grenouilles_triees[d], &grenouilles_triees[0]) <
	      distance(&grenouilles_triees[i], &grenouilles_triees[0]))
	    {
	      grenouilles_triees[d].x = grenouilles_triees[i].x;
	      grenouilles_triees[d].y = grenouilles_triees[i].y;
	      grenouilles_triees[d].numero = grenouilles_triees[i].numero;
	    }
	}
      else
	{
	  ++d;
	  grenouilles_triees[d].x = grenouilles_triees[i].x;
	  grenouilles_triees[d].y = grenouilles_triees[i].y;
	  grenouilles_triees[d].numero = grenouilles_triees[i].numero;
	}
    }
  return d+1; // taille du tableau restant
}

// trois points sont-ils dans le sens trigo ?
bool sens_trigo(const gren *g1, const gren *g2, const gren *g3)
{
  return (g1->x*g2->y - g1->y*g2->x + g1->y*g3->x
	  -g1->x*g3->y + g2->x*g3->y - g2->y*g3->x) > 0;
}

// calcul de l'enveloppe convexe
// le point de dpart est grenouilles[0], puis tri par angle croissant
int enveloppe_convexe(int d)
{
  int h = 2; // nombre de points sur l'enveloppe convexe
  int i=3; // point courant

  for (int j=0; j < (d<3?d:3); ++j)
    ec[j] = grenouilles_triees[j].numero;

  if (d<=3)
    {
      ec[d] = ec[0];
      return d+1;
    }
  while (i<d)
    {
      // si on tourne dans le bon sens
      if (sens_trigo(&grenouilles[ec[h-1]],
		     &grenouilles[ec[h]],
		     &grenouilles_triees[i]))
	{
	  ++h;
	  ec[h] = grenouilles_triees[i].numero;
	  ++i;
	}
      else
	--h;
    }

  // fermer la courbe
  ++h;
  ec[h] = ec[0];

  return h+1;
}


int main()
{
  int N; // nombre d'instances
  int fil[MAX_FROSH+1]; // pour stocker le parcours du fil
  int gnum, gnew; // numeros de grenouilles

  std::cin >> N;
  grenouilles[0].x = grenouilles[0].y = 0.0; // coordonnes du poteau
  grenouilles[0].numero = 0;

  for (int instance = 0; instance < N; ++instance)
    {
      if (instance != 0)
	printf("\n");
      // lecture de l'entre
      std::cin >> n;
      ref_x = ref_y = 0.0;
      for (int i=1; i<=n; ++i)
	{
	  std::cin >> grenouilles[i].x >> grenouilles[i].y;
	  grenouilles[i].numero = i;
	  // recherche du point le plus en bas  gauche
	  if ((grenouilles[i].x < ref_x) ||
	      ((grenouilles[i].x == ref_x) && (grenouilles[i].y < ref_y)))
	    {
	      ref_x = grenouilles[i].x;
	      ref_y = grenouilles[i].y;
	    }
	}

      // tri des grenouilles par rapport au point bas gauche
      memcpy(grenouilles_triees, grenouilles, (n+1)*sizeof(gren));
      qsort(grenouilles_triees, n+1, sizeof(gren), comparaison);

      // suppression des points aligns et doublons
      int d = suppression(); // d = nb pts dans tableau tri aprs suppression

      // calcul de l'enveloppe convexe
      int h = enveloppe_convexe(d); // h = nb pts dans ec
      // calcul du primtre de l'ec
      float perimetre_ec = 2.0; // initialis  2m pour faire le noeud
      for (int i=0; i<h-1; ++i)
	perimetre_ec += distance(&grenouilles[ec[i]], &grenouilles[ec[i+1]]);

      // si le poteau est dans l'ec, renvoyer le primtre
      int cpt=0;
      while ((cpt<h) && ec[cpt] != 0)
	++cpt;
      if (cpt < h)
	{
	  printf("%.2f\n", perimetre_ec);
	  continue;
	}

      // tri des grenouilles par rapport au poteau
      ref_x = ref_y = 0.0;
      memcpy(grenouilles_triees, grenouilles, (n+1)*sizeof(gren));
      qsort(grenouilles_triees, n+1, sizeof(gren), comparaison);
      d = suppression();

      // pour toutes les directions calculer la longueur de fil ncessaire
      float dist1[MAX_FROSH], dist2[MAX_FROSH];
      int nb_gren_int;
      int g=1;
      float fil_min = 2*distance(&grenouilles[0], &grenouilles[ec[0]])
	+ distance(&grenouilles[ec[1]], &grenouilles[ec[0]]);
      int num_min=0;
      for (int num = 0; num < h-1; ++num)
	// insertion de l'aller-retour entre ec[num] et ec[num+1]
	{
	  // parcours dans le sens trigo, en partant de ec[num]

	  // trouver la premire grenouille  l'intrieur
	  while (grenouilles_triees[g].numero != ec[num])
	    INCR(g);
	  INCR(g); // passer  la premire grenouille strictement intrieure

	  nb_gren_int = 1;
	  fil[0]=0; // poteau
	  fil[1] = ec[num];
	  dist1[0] = 0;
	  dist1[ec[num]] = distance(&grenouilles[0], &grenouilles[ec[num]]);

	  // boucle sur les grenouilles  l'intrieur
	  while ((gnum = grenouilles_triees[g].numero) != ec[num+1])
	    {
	      if (sens_trigo(&grenouilles[fil[nb_gren_int-1]],
			     &grenouilles[fil[nb_gren_int]],
			     &grenouilles[gnum]))
		{
		  dist1[gnum] = dist1[fil[nb_gren_int]] -
		    distance(&grenouilles[fil[nb_gren_int]], &grenouilles[0]) +
		    distance(&grenouilles[gnum], &grenouilles[0]) +
		    distance(&grenouilles[gnum],
			     &grenouilles[fil[nb_gren_int]]);
		  ++nb_gren_int;
		  fil[nb_gren_int] = gnum;
		  INCR(g);
		}
	      else
		--nb_gren_int;
	    }

	  // parcours maintenant dans le sens horaire, en partant de ec[num+1]

	  // trouver la premire grenouille  l'intrieur
	  DECR(g); // passer  la grenouille prcdente

	  nb_gren_int = 1;
	  fil[0]=0; // poteau
	  fil[1] = ec[num+1];
	  dist2[0]=0;
	  dist2[ec[num+1]]=distance(&grenouilles[0],&grenouilles[ec[num+1]]);

	  // boucle sur les grenouilles  l'intrieur
	  while ((gnum = grenouilles_triees[g].numero) != ec[num])
	    {
	      if (!sens_trigo(&grenouilles[fil[nb_gren_int-1]],
			      &grenouilles[fil[nb_gren_int]],
			      &grenouilles[gnum]))
		{
		  dist2[gnum] = dist2[fil[nb_gren_int]] -
		    distance(&grenouilles[fil[nb_gren_int]], &grenouilles[0]) +
		    distance(&grenouilles[gnum], &grenouilles[0]) +
		    distance(&grenouilles[gnum],
			     &grenouilles[fil[nb_gren_int]]);
		  ++nb_gren_int;
		  fil[nb_gren_int] = gnum;
		  DECR(g);
		}
	      else
		--nb_gren_int;
	    }
	  float distance_min =
	    2*distance(&grenouilles[0], &grenouilles[ec[num]]) +
	    distance(&grenouilles[ec[num+1]], &grenouilles[ec[num]]);

	  while ((gnum = grenouilles_triees[g].numero) != ec[num+1])
	    {
	      INCR(g);
	      gnew = grenouilles_triees[g].numero;
	      if (distance_min > dist1[gnum]+dist2[gnew])
		distance_min = dist1[gnum]+dist2[gnew];
	    }
	  distance_min -=
	    distance(&grenouilles[ec[num]], &grenouilles[ec[num+1]]);
	  if (distance_min < fil_min)
	    {
	      fil_min = distance_min;
	      num_min = num;
	    }
	}

      printf("%.2f\n", perimetre_ec+fil_min);

    }

}
